/* 
 * FILE --- fuzzy_sys_ex3.cc
 *
 * PURPOSE --- To provide an example use of the class for 4'th
 * order Runge-Kutta simulation, RK4_SIM, and the class for implementing
 * n-input 1-output Takagi-Sugeno (TS) fuzzy system, FUZZY_TS_SYS.
 *
 * Implements the main function of a class for performing a simulation of
 * a simple vehicle with point mass dynamics controlled by a direct adaptive  
 * TS fuzzy controller.
 * The fuzzy controller is implemented as a part of the class for the vehicle
 * dynamics - VEH_SIM_FUZZY_TS. 
 *
 */

// Include Files
#include <stdlib.h>
#include <stdio.h>

#include "veh_sim_fuzzy_ts.hh"


void main(void) 
{
  double Ts = 0.01;             // Sampling time
  double y[3] = {-0.9*45.0,45.0,0.0};  // Initial conditions    
  double t0 = 0.0;              // Initial time 
  double tmax = 200.0;              // Total Simulation period

  // SISO fuzzy controller gains
  double input_gain = 1/15.0, output_gain = 2000.0;

  // # of simulation steps to be performed by the RK4 algorith
  // in one sampling interval
  int sim_per_cycle = 5;  

  // Create the vehicle object
  VEH_SIM_FUZZY_TS my_vehicle(Ts,sim_per_cycle,t0,(double *)y);

  FILE *fp;  // To save data

  if (!(fp = fopen("veh_fuzzy_ts_out2.dat","w"))) 
    {
      printf("Can't open file.\n");
      exit(1);
    }

  // Set the TS fuzzy controller input gains
  my_vehicle.fuzzy_ts_sys->set_input_gains(&input_gain);

  // Set the coefficients to the TS inputs of the fuzzy controller as 0.0
  my_vehicle.fuzzy_ts_sys->set_ts_coeff(0.0);

  // Set the output gain for the TS fuzzy controller
  my_vehicle.output_gain = output_gain;

  // Simulate For tmax seconds.
  while(my_vehicle.current_time < tmax) 
    {
      // Simulate one step
      my_vehicle.simulate();

      // Update the output parameters (TS coefficients)
      // my_vehicle.ctrl_input[2] = output error
      my_vehicle.fuzzy_ts_sys->update_ts_coeff(my_vehicle.ctrl_input[2]);

      // Save data to file
      fprintf(fp,"%f %f %f %f %f %f %f\n", my_vehicle.current_time,
	      my_vehicle.plant_state[0], my_vehicle.plant_state[1],
	      my_vehicle.plant_state[2], my_vehicle.ctrl_input[0],
	      my_vehicle.ctrl_input[1], my_vehicle.ctrl_input[2]);
    }

  fclose(fp);
}



